@@ -46,6 +46,8 @@
 #include <stdlib.h>
 #include <stdio.h>
 
+#include <limits.h>
+
 #ifndef ARRAY_LEN
 #define ARRAY_LEN(a) sizeof((a))/sizeof((a)[0])
 #endif
@@ -237,36 +239,81 @@ static void test_power(void)
     hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){2., 0.5});
     hdrl_image_pow_scalar(himg, (hdrl_value){2., 0.});
     v = hdrl_image_get_pixel(himg, 1, 1, NULL);
-    cpl_test_rel(v.data, 4., DBL_EPSILON);
-    cpl_test_rel(v.error, 2.0, DBL_EPSILON);
+    cpl_test_rel(v.data, 4., HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 2.0, HDRL_EPS_ERROR);
 
     hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){2., 0.5});
     hdrl_image_pow_scalar(himg, (hdrl_value){4., 0.});
     v = hdrl_image_get_pixel(himg, 1, 1, NULL);
-    cpl_test_rel(v.data, 16, DBL_EPSILON);
-    cpl_test_rel(v.error, 16, DBL_EPSILON);
+    cpl_test_rel(v.data, 16, HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 16, HDRL_EPS_ERROR);
 
     hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){2., 0.5});
     hdrl_image_pow_scalar(himg, (hdrl_value){-1., 0.});
     v = hdrl_image_get_pixel(himg, 1, 1, NULL);
-    cpl_test_rel(v.data, 0.5, DBL_EPSILON);
-    cpl_test_rel(v.error, 0.125, DBL_EPSILON);
+    cpl_test_rel(v.data, 0.5, HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 0.125, HDRL_EPS_ERROR);
 
     hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){2., 0.5});
     hdrl_image_pow_scalar(himg, (hdrl_value){-2., 0.});
     v = hdrl_image_get_pixel(himg, 1, 1, NULL);
-    cpl_test_rel(v.data, 0.25, DBL_EPSILON);
-    cpl_test_rel(v.error, 0.125, DBL_EPSILON); /* yes the same as ^-1 */
+    cpl_test_rel(v.data, 0.25, HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 0.125, HDRL_EPS_ERROR); /* yes the same as ^-1 */
 
     hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){2., 0.5});
     hdrl_image_pow_scalar(himg, (hdrl_value){-4., 0.});
     v = hdrl_image_get_pixel(himg, 1, 1, NULL);
-    cpl_test_rel(v.data, 0.0625, DBL_EPSILON);
-    cpl_test_rel(v.error, 0.0625, DBL_EPSILON);
+    cpl_test_rel(v.data, 0.0625, HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 0.0625, HDRL_EPS_ERROR);
+
+    hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){2., 0.3});
+    hdrl_image_pow_scalar(himg, (hdrl_value){-3., 0.0});
+    v = hdrl_image_get_pixel(himg, 1, 1, NULL);
+    cpl_test_rel(v.data, 1.0/8.0, HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 0.05625, HDRL_EPS_ERROR);
 
     hdrl_image_delete(himg);
 }
 
+static void test_exp(void)
+{
+    hdrl_value v;
+    hdrl_image * himg = hdrl_image_new(1, 1);
+    hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){2., 0.0});
+    hdrl_image_exp_scalar(himg, (hdrl_value){2., 0.5});
+    v = hdrl_image_get_pixel(himg, 1, 1, NULL);
+    cpl_test_rel(v.data, 4., HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 2.0, HDRL_EPS_ERROR);
+
+
+    hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){4., 0.});
+    hdrl_image_exp_scalar(himg, (hdrl_value){2., 0.5});
+    v = hdrl_image_get_pixel(himg, 1, 1, NULL);
+    cpl_test_rel(v.data, 16, HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 16, HDRL_EPS_ERROR);
+
+    hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){-1., 0.});
+    hdrl_image_exp_scalar(himg, (hdrl_value){2., 0.5});
+    v = hdrl_image_get_pixel(himg, 1, 1, NULL);
+    cpl_test_rel(v.data, 0.5, HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 0.125, HDRL_EPS_ERROR);
+
+    hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){-2., 0.});
+    hdrl_image_exp_scalar(himg, (hdrl_value){2., 0.5});
+    v = hdrl_image_get_pixel(himg, 1, 1, NULL);
+    cpl_test_rel(v.data, 0.25, HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 0.125, HDRL_EPS_ERROR);
+
+    hdrl_image_set_pixel(himg, 1, 1, (hdrl_value){-4., 0.});
+    hdrl_image_exp_scalar(himg, (hdrl_value){2., 0.5});
+    v = hdrl_image_get_pixel(himg, 1, 1, NULL);
+    cpl_test_rel(v.data, 0.0625, HDRL_EPS_DATA);
+    cpl_test_rel(v.error, 0.0625, HDRL_EPS_ERROR);
+
+    hdrl_image_delete(himg);
+}
+
+
 static void test_copy(void)
 {
     hdrl_image * dst = hdrl_image_new(50, 50);
@@ -426,7 +473,14 @@ static void test_reduce(void)
         hdrl_image_set_pixel(a, 3, 1, (hdrl_value){3., 2.5});
         hdrl_image_reject(a, 1, 1);
 
+        m = hdrl_image_get_sum(NULL);
+        cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+        cpl_test(isnan(m.data) && isnan(m.error));
+
         m = hdrl_image_get_sum(a);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test(!isnan(m.data) && !isnan(m.error));
+
         cpl_test_abs(m.data, 5., HDRL_EPS_DATA);
         cpl_test_abs(m.error, sqrt(1.5 * 1.5 + 2.5 * 2.5), HDRL_EPS_ERROR * 3);
 
@@ -437,29 +491,139 @@ static void test_reduce(void)
     }
 }
 
-
 static inline void test_buffer(void)
 {
-    hdrl_buffer * buf = hdrl_buffer_new();
-    hdrl_imagelist * hl = hdrl_imagelist_new();
-    for (size_t i = 0; i < 900; i++) {
-        hdrl_image * img =hdrl_image_new_from_buffer(500, 500, buf);
+    hdrl_buffer    *buf = hdrl_buffer_new();
+    hdrl_imagelist *hl  = hdrl_imagelist_new();
+
+#if (__WORDSIZE == 64)
+
+    cpl_size iMax    = 100;     /* Previously = 900 */
+    cpl_size sizeImg =  64;		/* Previously = 500 */
+
+    for (cpl_size i = 0; i < iMax; i++) {
+        hdrl_image * img = hdrl_image_new_from_buffer(sizeImg, sizeImg, buf);
         hdrl_imagelist_set(hl, img, i);
         hdrl_image_add_scalar(img, (hdrl_value){1, 1});
     }
     cpl_msg_warning(cpl_func, "alloc done");
 
-    hdrl_image * m;
-    cpl_image * c;
+    hdrl_image *m;
+    cpl_image  *c;
     hdrl_imagelist_collapse_mean(hl, &m, &c);
     hdrl_image_delete(m);
     cpl_image_delete(c);
     cpl_msg_warning(cpl_func, "collapse done");
 
+
+#endif
+
+
     hdrl_imagelist_delete(hl);
+
     hdrl_buffer_delete(buf);
 }
 
+static void test_create(void)
+{
+	/* Create reference images */
+	size_t nx = 10;
+	size_t ny = 100;
+	hdrl_image *a = hdrl_image_new(nx, ny);
+	hdrl_image *b = hdrl_image_new(nx, ny);
+
+	/* Reject value */
+	hdrl_image_reject_value(a, CPL_VALUE_NAN);
+	hdrl_image_reject_value(b, CPL_VALUE_NAN);
+
+	/* Add information */
+	hdrl_value hValue = {2., 0.5};
+	hdrl_image_add_scalar(a, hValue);
+	hdrl_image_add_scalar(b, hValue);
+
+	hdrl_image *new1;
+	hdrl_image *new2;
+	hdrl_image *new3;
+	hdrl_image *new4;
+	hdrl_image *new5;
+	hdrl_image *new6;
+
+	/* Basic operations */
+
+	new1 = hdrl_image_add_image_create(NULL, b);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new1);
+	new1 = hdrl_image_add_image_create(a, NULL);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new1);
+	new1 = hdrl_image_add_image_create(a, b);
+	cpl_test_error(CPL_ERROR_NONE);
+	cpl_test_nonnull(new1);
+
+	new2 = hdrl_image_sub_image_create(NULL, b);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new2);
+	new2 = hdrl_image_sub_image_create(a, NULL);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new2);
+	new2 = hdrl_image_sub_image_create(a, b);
+	cpl_test_error(CPL_ERROR_NONE);
+	cpl_test_nonnull(new2);
+
+
+	new3 = hdrl_image_mul_image_create(NULL, b);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new3);
+	new3 = hdrl_image_mul_image_create(a, NULL);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new3);
+	new3 = hdrl_image_mul_image_create(a, b);
+	cpl_test_error(CPL_ERROR_NONE);
+	cpl_test_nonnull(new3);
+
+
+	new4 = hdrl_image_div_image_create(NULL, b);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new4);
+	new4 = hdrl_image_div_image_create(a, NULL);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new4);
+	new4 = hdrl_image_div_image_create(a, b);
+	cpl_test_error(CPL_ERROR_NONE);
+	cpl_test_nonnull(new4);
+
+
+	/* Pow and Exp operations */
+
+	new5 = hdrl_image_pow_scalar_create(NULL, hValue);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new5);
+	new5 = hdrl_image_pow_scalar_create(a, hValue);
+	cpl_test_error(CPL_ERROR_NONE);
+	cpl_test_nonnull(new5);
+
+	new6 = hdrl_image_exp_scalar_create(NULL, hValue);
+	cpl_test_error(CPL_ERROR_NULL_INPUT);
+	cpl_test_null(new6);
+	new6 = hdrl_image_exp_scalar_create(a, hValue);
+	cpl_test_error(CPL_ERROR_NONE);
+	cpl_test_nonnull(new6);
+
+
+	/* Clean up */
+    hdrl_image_delete(a);
+    hdrl_image_delete(b);
+
+    hdrl_image_delete(new1);
+    hdrl_image_delete(new2);
+    hdrl_image_delete(new3);
+    hdrl_image_delete(new4);
+
+    hdrl_image_delete(new5);
+    hdrl_image_delete(new6);
+}
+
+
 /*----------------------------------------------------------------------------*/
 /**
  @brief   Unit tests of hdrl_image
@@ -471,11 +635,15 @@ int main(void)
 
     test_basic();
     test_power();
+    test_exp();
     test_copy();
     test_insert();
     test_extract();
     test_reduce();
-    //test_buffer();
+
+    test_create();
+
+    test_buffer();
 
     return cpl_test_end(0);
 }
