@@ -769,7 +769,8 @@ static cpl_imagelist * naco_imagelist_fit_polynomial(
             }
         }
 
-        cpl_matrix_delete(mv);
+        cpl_free(pv); /* Free explicitly - to avoid cppcheck false positive */
+        (void)cpl_matrix_unwrap(mv);
         (void)cpl_matrix_unwrap(mb);
 
         if (nsing == npix) {
