@@ -321,6 +321,7 @@ static cpl_table * naco_img_zpoint_reduce(cpl_propertylist       * qclist,
                                           const char             * flat,
                                           cpl_image             ** pcheck_im)
 {
+    cpl_errorstate           prestate = cpl_errorstate_get();
     const cpl_propertylist * plist
         = irplib_framelist_get_propertylist_const(rawframes, 0);
     cpl_image              * flat_im = NULL;
@@ -337,6 +338,7 @@ static cpl_table * naco_img_zpoint_reduce(cpl_propertylist       * qclist,
     cpl_vector             * sigmas = NULL;
     cpl_table              * out_tab = NULL;
     cpl_vector             * zpoints = NULL;
+    cpl_stats              * fstats = NULL;
     double                   dit;
     double                   off_x0 = DBL_MAX; /* Avoid uninit warning */
     double                   off_y0 = DBL_MAX; /* Avoid uninit warning */
@@ -351,6 +353,7 @@ static cpl_table * naco_img_zpoint_reduce(cpl_propertylist       * qclist,
     double                   avg_zp = 0.0;
     int                      nok_zp = 0;
     int                      nok_strehl = 0;
+    cpl_error_code           code;
     int                      i;
 
     /* QC Parameters */
@@ -431,12 +434,20 @@ static cpl_table * naco_img_zpoint_reduce(cpl_propertylist       * qclist,
 
     /* Load the the flatfield if one is provided */
     if (flat != NULL) {
-        cpl_msg_info(cpl_func, "Loading the flat field from %s", flat);
         flat_im = cpl_image_load(flat, CPL_TYPE_FLOAT, 0, 0);
 
         if (flat_im == NULL) {
-            cpl_msg_error(cpl_func, "Could not load flat field");
+            cpl_msg_error(cpl_func, "Could not load flat field %s", flat);
             skip_if(1);
+        } else {
+            fstats = cpl_stats_new_from_image(flat_im, CPL_STATS_ALL);
+            if (cpl_stats_get_min(fstats) <= 0.0) {
+                cpl_msg_warning(cpl_func, "Setting non-positive (down to %g) "
+                                "flat field pixels to 1",
+                                cpl_stats_get_min(fstats));
+                bug_if(cpl_image_threshold(flat_im, 0.0, DBL_MAX, 1.0,
+                                           DBL_MAX));
+            }
         }
     }
 
@@ -549,6 +560,8 @@ static cpl_table * naco_img_zpoint_reduce(cpl_propertylist       * qclist,
 
         if (i > 0) {
             /* Refine the positions */
+            const double opos_x = pos_x;
+            const double opos_y = pos_y;
             const int llx = (int)pos_x - naco_img_zpoint_config.sx;
             const int urx = (int)pos_x + naco_img_zpoint_config.sx;
             const int lly = (int)pos_y - naco_img_zpoint_config.sy;
@@ -557,6 +570,9 @@ static cpl_table * naco_img_zpoint_reduce(cpl_propertylist       * qclist,
             pos_y = cpl_image_get_centroid_y_window(imdiff, llx, lly, urx, ury);
             error_if (0, cpl_error_get_code(), "Could not refine the positions "
                       "of difference image %d", i+1);
+            cpl_msg_info(cpl_func, "For difference image %d refined object "
+                         "position (x,y): (%g,%g) -> (%g,%g)", i,
+                         opos_x, opos_y, pos_x, pos_y);
         }
 
         bug_if(0);
@@ -575,13 +591,25 @@ static cpl_table * naco_img_zpoint_reduce(cpl_propertylist       * qclist,
         }
 
         /* Compute the flux/strehl in the current image */
-        skip_if (naco_img_zpoint_reduce_one(out_tab, zpoints, i, imdiff,
-                                            parlist, filter,
-                                            pos_x, pos_y, dit,
-                                            &str, &str_err, &pe, &fl, &bg_no));
+        code = naco_img_zpoint_reduce_one(out_tab, zpoints, i, imdiff,
+                                          parlist, filter,
+                                          pos_x, pos_y, dit,
+                                          &str, &str_err, &pe, &fl, &bg_no);
+        if (code) {
+            cpl_msg_warning(cpl_func, "Strehl computation for difference image "
+                            "%d at (x,y)=(%g,%g) failed:", i+1, pos_x, pos_y);
+            cpl_errorstate_dump(prestate, CPL_FALSE,
+                                cpl_errorstate_dump_one_warning);
+            cpl_errorstate_set(prestate);
+            continue;
+        } else if (str <= 0.0 || str >= 1.0) {
+            cpl_msg_info(cpl_func, "Excluding from statistics non-physical "
+                         "Strehl ratio for difference image %d at (x,y)=(%g,%g)"
+                         ": %g", i+1, pos_x, pos_y, str);
+            continue;
+        }
 
         /* Compute the averages of the results for the strehl */
-        if (str <= 0.0 || str >= 1.0) continue;
         nok_strehl++;
 
         str_sum     += str;
@@ -643,6 +671,7 @@ static cpl_table * naco_img_zpoint_reduce(cpl_propertylist       * qclist,
 
     end_skip;
     
+    cpl_stats_delete(fstats);
     cpl_image_delete(rawimage);
     cpl_image_delete(imdiff);
     cpl_image_delete(flat_im);
@@ -770,7 +799,7 @@ static cpl_error_code naco_img_zpoint_check_im(cpl_image          ** self,
   @param    zptab        Table to be filled with zero point information
   @param    zpoints      Vector with an element to be set to zero point
   @param    idiff        Index of row element to modify
-  @param    ima          The input image
+  @param    self         The input image
   @param    parlist      The recipe parameter list
   @param    filter       The filter
   @param    pos_x        The object x position
@@ -789,7 +818,7 @@ static
 cpl_error_code naco_img_zpoint_reduce_one(cpl_table       * zptab,
                                           cpl_vector      * zpoints,
                                           int               idiff,
-                                          const cpl_image * ima,
+                                          const cpl_image * self,
                                           const cpl_parameterlist * parlist,
                                           const char      * filter,
                                           double            pos_x,
@@ -816,10 +845,10 @@ cpl_error_code naco_img_zpoint_reduce_one(cpl_table       * zptab,
     r = naco_img_zpoint_config.phot_star_radius/naco_img_zpoint_config.pscale;
     r1 = naco_img_zpoint_config.phot_bg_r1/naco_img_zpoint_config.pscale;
     r2 = naco_img_zpoint_config.phot_bg_r2/naco_img_zpoint_config.pscale;
-    background = irplib_strehl_ring_background(ima, pos_x, pos_y, r1, r2,
+    background = irplib_strehl_ring_background(self, pos_x, pos_y, r1, r2,
             IRPLIB_BG_METHOD_AVER_REJ);
     skip_if(0);
-    *pstar_flux = irplib_strehl_disk_flux(ima, pos_x, pos_y, r, background);
+    *pstar_flux = irplib_strehl_disk_flux(self, pos_x, pos_y, r, background);
     skip_if(0);
 
     /* Get lam and dlam from the filter name for the Strehl computation */
@@ -827,16 +856,17 @@ cpl_error_code naco_img_zpoint_reduce_one(cpl_table       * zptab,
                  "Cannot get filter infos [%s]", filter);
     
     /* Compute the strehl */
-    irplib_check(naco_strehl_compute(ima, parlist, RECIPE_STRING, lam, dlam,
+    irplib_check(naco_strehl_compute(self, parlist, RECIPE_STRING, lam, dlam,
                                      pos_x, pos_y,
                                      naco_img_zpoint_config.pscale,
                                      pstrehl, pstrehl_err, &star_bg,
                                      pstar_peak, pstar_flux,
                                      &psf_peak, &psf_flux, pbg_noise),
-                 "Could not compute the Strehl ratio");
+                 "Could not compute the Strehl ratio. r=%g <= r1=%g < r2=%g",
+                 r, r1, r2);
 
     /* FWHM_X / FWHM_Y */
-    if (cpl_image_get_fwhm(ima, (int)pos_x, (int)pos_y, &fwhm_x, &fwhm_y)) {
+    if (cpl_image_get_fwhm(self, (int)pos_x, (int)pos_y, &fwhm_x, &fwhm_y)) {
         irplib_error_recover(cleanstate, "Could not compute FWHM for image %d:",
                           idiff+1);
         fwhm_x = fwhm_y = 0.0;
